@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-#     $Id: add_html_refs.sh,v 1.6 2001/04/27 21:44:27 moniot Exp $
+#     $Id: add_html_refs.sh,v 1.7 2002/08/18 18:43:21 moniot Rel $
 #
 # Script to insert links from all occurrences of each ftnchek option
 # to the main description of the option in ftnchek.html.
@@ -39,7 +39,7 @@ ftnchek -help | \
 # where the options occur in the entire text.
 # The third sed puts hot-links in for the -[no]options in SYNOPSIS section
 # that are not recognized by the second sed.
-tr '\a' '"' |
+tr '\007' '"' |
 sed -e '/^Content-type:/d' \
     -e '/<H2>OPTIONS<\/H2>/,/<H2>.*<\/H2>/s,^<DT><B>-\([a-z][-a-z0-9]*\)[^<]*</B>,<A NAME="\1"></A>&,' | \
 sed -f option_sub.sed  | \
