@@ -1,4 +1,4 @@
-/* $Id: argcheck.c,v 1.9 2001/08/26 16:24:09 moniot Exp $
+/* $Id: argcheck.c,v 1.10 2002/12/15 19:43:15 moniot Rel $
 
 	Routines to check subprogram type and argument agreement
 
@@ -109,20 +109,28 @@ arg_array_cmp(name,args1,args2)
 				   matches any numeric or logical type
 				   but not  character.  The single/double
 				   match will be deferred to size check. */
-	      int cmptype1= (t1==type_HOLLERITH && t2!=type_STRING)?
-				t2:type_category[t1];
-	      int cmptype2= (t2==type_HOLLERITH && t1!=type_STRING)?
-				t1:type_category[t2];
+	      int cmptype1, cmptype2;
 
 		/* If -portability, do not translate default sizes so
 		   they will never match explicit sizes. */
-	      if(!(port_mixed_size || local_wordsize==0)) {
+	      if( port_mixed_size || local_wordsize==0 ) {
+		cmptype1 = (t1==type_HOLLERITH && t2!=type_STRING)?
+				t2:t1;
+		cmptype2 = (t2==type_HOLLERITH && t1!=type_STRING)?
+				t1:t2;
+	      }
+	      else {
+		cmptype1 = (t1==type_HOLLERITH && t2!=type_STRING)?
+				t2:type_category[t1];
+		cmptype2 = (t2==type_HOLLERITH && t1!=type_STRING)?
+				t1:type_category[t2];
 		if(defsize1)
 		  s1 = type_size[t1];
 		if(defsize2)
 		  s2 = type_size[t2];
 	      }
 
+
 	      if(s1 < 0 || s2 < 0) { /* char size_ADJUSTABLE or UNKNOWN */
 		s1 = s2 = size_DEFAULT;	/* suppress warnings on size */
 		defsize1 = defsize2 = TRUE;
