@@ -1,4 +1,4 @@
-/* $Id: calltree.c,v 1.4 2001/08/26 16:24:11 moniot Exp $
+/* $Id: calltree.c,v 1.8 2002/11/20 23:13:15 moniot Rel $
 
    Routines for producing call trees and cross-reference lists
 
@@ -77,6 +77,113 @@ PROTO(PRIVATE void print_modules,( unsigned n, Gsymtab *list[] ));
 PRIVATE int com_tree_error;
 PRIVATE int numvisited;
 
+/**********************************************************************************
+*
+* append_extension (imported as-is from ftnchek.c ftnchek 3.1.1)
+*
+* MODE_DEFAULT_EXT: Adds extension to file name s if
+*                  none is present, and returns a pointer to the
+*                  new name.  If extension was added, space is allocated
+*                  for the new name.  If not, simply  returns pointer
+*                  to original name.  
+* MODE_REPLACE_EXT: same, except given extension replaces given one if any.
+*
+* Returns char * to newly allocated name string.
+*
+**********************************************************************************/
+#define MODE_DEFAULT_EXT 1
+#define MODE_REPLACE_EXT 2
+PRIVATE char *
+#if HAVE_STDC
+append_extension( char *s, char *ext, int mode )
+#else                        /* K&R style */
+append_extension( s, ext, mode )
+     char *s, *ext;
+     int mode;
+#endif                       /* HAVE_STDC */
+   {
+   int i, len;
+   char *newname;
+#ifdef OPTION_PREFIX_SLASH                      /* set len=chars to NUL or start
+                                                *  of /opt */
+   for ( len = 0; s[len] != '\0' && s[len] != '/'; len++ )
+      continue;
+#else
+   len = ( unsigned ) strlen( s );
+#endif
+   /*
+   *  Search backwards till find the dot, but do not search past directory
+   *  delimiter 
+   */
+   for ( i = len - 1; i > 0; i-- )
+      {
+      if ( s[i] == '.'
+#ifdef UNIX
+           || s[i] == '/'
+#endif
+#ifdef VMS
+           || s[i] == ']' || s[i] == ':'
+#endif
+#ifdef MSDOS
+           || s[i] == '\\' || s[i] == ':'
+#endif
+          )
+         break;
+      }
+
+   if ( mode == MODE_REPLACE_EXT )
+      {
+      if ( s[i] == '.' )                        /* declare length = up to the dot */
+         len = i;
+      newname =
+         ( char * )
+         malloc( ( unsigned ) ( len + ( unsigned ) strlen( ext ) + 1 ) );
+      ( void ) strncpy( newname, s, len );
+      ( void ) strcpy( newname + len, ext );
+      }
+   else
+      {                                         /* MODE_DEFAULT_EXT */
+#ifdef OPTION_PREFIX_SLASH
+      /*
+      *  create new string if new ext or trailing /option 
+      */
+      if ( s[i] != '.' || s[len] != '\0' )
+         {
+         if ( s[i] != '.' )
+            {                                   /* no extension given */
+            newname = ( char * ) malloc( ( unsigned ) ( len +
+                                                        ( unsigned ) strlen( ext )
+                                                        + 1 ) );
+            ( void ) strncpy( newname, s, len );
+            ( void ) strcpy( newname + len, ext );
+            }
+         else
+            {                                   /* extension given but /option
+                                                *  follows */
+            newname = ( char * ) malloc( ( unsigned ) ( len + 1 ) );
+            ( void ) strncpy( newname, s, len );
+            }
+         }
+#else
+      if ( s[i] != '.' )
+         {
+         newname = ( char * ) malloc( ( unsigned ) ( len +
+                                                     ( unsigned ) strlen( ext ) +
+                                                     1 ) );
+         ( void ) strcpy( newname, s );
+         ( void ) strcat( newname, ext );
+         }
+#endif
+      else
+         {
+         newname = s;                           /* use as is */
+         }
+      }
+
+   return newname;
+   }
+
+
 void
 com_xref_list(VOID)	/* Print cross-reference list of com blocks */
 {
@@ -449,21 +556,91 @@ visit_child(gsymt,level)
 {
   static char fmt[]="%000s";	/* Variable format for indenting names */
   ChildList *child_list;
+  static int terminate_href = 0;
+  char *fname=NULL;
+  ArgListHeader *arghdr;
 
 
   if(print_call_tree) {
+     if ( htmlcalltree_fd ) {
+				/* Look up defn arglist entry to find the
+				   filename where this guy is defined.
+				 */
+	for ( arghdr = gsymt->info.arglist; arghdr; arghdr=arghdr->next )
+	{
+	   if ( arghdr->is_defn && arghdr->filename )
+	   {
+	      fname = append_extension( arghdr->filename,
+					DEF_HTML_EXTENSION, MODE_REPLACE_EXT );
+	   }
+	}
+	if ( terminate_href )
+            {
+            terminate_href = 0;
+            ( void ) fprintf( htmlcalltree_fd, "</A>" );
+            }
+         ( void ) fprintf( htmlcalltree_fd, "\n" );
+     }
     (void)fprintf(list_fd,"\n");
     if(level > 0) {
       (void)sprintf(fmt,"%%%ds",level*4); /* indent 4 spaces per nesting level */
       (void)fprintf(list_fd,fmt,"");
+      if ( htmlcalltree_fd )
+         {           
+         if ( fname )
+            {
+            if ( ! gsymt->internal_entry )
+               ( void ) fprintf( htmlcalltree_fd, "%*.*s<A href=\"%s#%s\">",
+                                 level * 4, level * 4, " ",
+                                 fname, gsymt->name );
+            else
+               ( void ) fprintf( htmlcalltree_fd, "%*.*s<A href=\"%s#%s\">",
+                                 level * 4, level * 4, " ",
+                                 fname, gsymt->link.module->name );
+            terminate_href = 1;
+            }
+         else
+            {
+            ( void ) fprintf( htmlcalltree_fd, "%*.*s", level * 4, level * 4,
+                              " " );
+            terminate_href = 0;
+            }
+         }      
     }
     if(gsymt->internal_entry)
+      {
       (void)fprintf(list_fd,"%s entry ",gsymt->link.module->name);
+         if ( htmlcalltree_fd )
+            ( void ) fprintf( htmlcalltree_fd, "%s entry ",
+                              gsymt->link.module->name );      
+      }
     (void)fprintf(list_fd,"%s",gsymt->name);
+    if ( htmlcalltree_fd )
+       {
+       if ( level == 0 )
+          {
+          if ( gsymt->internal_entry )
+            {
+	       if(fname)
+		  ( void ) fprintf( htmlcalltree_fd, "<A href=\"%s#%s\">%s",
+                              fname, gsymt->link.module->name, gsymt->name );
+            }
+          else
+            {  
+            ( void ) fprintf( htmlcalltree_fd, "<A href=\"%s#%s\">%s",
+                              fname, gsymt->name, gsymt->name );
+            }
+          terminate_href = 1;
+          }
+       else
+          ( void ) fprintf( htmlcalltree_fd, "%s", gsymt->name );
+       }
+
+    if(fname)
+       free( fname );
+
   }
 
-
-
 				/* Visit its unvisited children.  Note
 				   that children of internal entry are
 				   taken as those of its superior module.
@@ -476,7 +653,18 @@ visit_child(gsymt,level)
 				   has some. */
   if(call_tree_prune && gsymt->visited) {
     if(print_call_tree && child_list != NULL)
+      {
       (void)fprintf(list_fd," (see above)");
+         if ( htmlcalltree_fd )
+            {
+            if ( terminate_href )
+               { 
+               ( void )fprintf( htmlcalltree_fd, "</A>" ); 
+               terminate_href=0; 
+               }
+            ( void ) fprintf( htmlcalltree_fd, " (see above)" );
+            }
+      }      
   }
   else {
 				/* Mark node as visited */
@@ -488,12 +676,26 @@ visit_child(gsymt,level)
     else
       gsymt->visited_somewhere = TRUE;
 
+   if ( print_call_tree )
+      {
+      if ( terminate_href )
+         {
+         ( void ) fprintf( htmlcalltree_fd, "</A>" );
+         terminate_href = 0;
+         }
+      }
+
     ++level;			/* move to next level */
     while(child_list != NULL) {
       visit_child(child_list->child,level);
       child_list = child_list->next;
     }
   }
+  if ( terminate_href )
+     {
+     ( void ) fprintf( htmlcalltree_fd, "</A>" );
+     terminate_href = 0;
+     }
 }
 
 /*** visit_child_reflist
