@@ -16,11 +16,18 @@
 		/* If stdlib.h exists, use it to declare malloc and friends */
 #define HAVE_STDLIB_H	1
 
+		/* Use strings.h for strcasecmp() and strncasecmp() */
+#define HAVE_STRINGS_H	1
+
 		/* Either memset or bzero is needed in forlex.c: memset
 		   is preferred.
 		 */
 #define HAVE_MEMSET	1
 #define HAVE_BZERO	0
 
-
-
+		/* Some compilers don't have strcasecmp but have stricmp
+		   or strcmpi instead.  Set up suitable defines here.
+		*/
+#define HAVE_STRCASECMP		0
+#define HAVE_STRICMP		0
+#define HAVE_STRCMPI		0
