@@ -31,7 +31,7 @@ as the "MIT License."
 
 /* I. */
 
-/* $Id: exprtype.c,v 1.12 2001/10/07 22:58:04 moniot Exp $
+/* $Id: exprtype.c,v 1.15 2005/02/07 00:38:01 moniot Exp $
 
 	Routines to propagate datatype through expressions.
 
@@ -51,6 +51,7 @@ as the "MIT License."
 #include "ftnchek.h"
 #define EXPRTYPE
 #include "symtab.h"
+#include "symutils.h"
 #include "tokdefs.h"
 
 
@@ -225,16 +226,20 @@ binexpr_type(term1,op,term2,result)
     int I_logop_I=FALSE;		/* for f90_mixed_type warning */
 
     if( ! is_computational_type(type1) ) {
+      if( misc_warn ) {
 		syntax_error(term1->line_num,term1->col_num,
 			"numeric or character quantity expected:");
 		report_type(term1);
-		result_type = E;
+      }
+      result_type = E;
     }
     else if( ! is_computational_type(type2) ) {
+      if( misc_warn ) {
 		syntax_error(term2->line_num,term2->col_num,
 			"numeric or character quantity expected:");
 		report_type(term2);
-		result_type = E;
+      }
+      result_type = E;
     }
     else {
 	switch(opclass) {
@@ -280,7 +285,7 @@ binexpr_type(term1,op,term2,result)
 			/* Intrinsic function argument list: no promotion
 			   across type categories.  Accept matching type
 			   categories: size match will be checked later. */
-	    case ',':
+	    case ',': /* INTRINS_ARGS */
 		if( type_category[type1] != type_category[type2] )
 		  result_type = E;
 		else if(type1 == S)
@@ -299,15 +304,16 @@ binexpr_type(term1,op,term2,result)
 
 	if( (type1 != E && type2 != E) ) {
 	    if( result_type == E) {
+	      if(INTRINS_ARGS) {
 		syntax_error(op->line_num,op->col_num,
-			"type mismatch");
-		if(INTRINS_ARGS) {
-		  msg_tail("between intrinsic function arguments:");
-		}
-		else {
-		  msg_tail("in expression:");
-		}
+		       "type mismatch between intrinsic function arguments:");
 		report_mismatch(term1,op,term2);
+	      }
+	      else if( misc_warn ) {
+		syntax_error(op->line_num,op->col_num,
+			     "operands cannot be combined in expression:");
+		report_mismatch(term1,op,term2);
+	      }
 	    }
 	    else if(result_type >= W) {	/* W result */
 				/* F90 warning suppressed for numeric exprs */
@@ -338,8 +344,7 @@ binexpr_type(term1,op,term2,result)
 
 				/* Figure out the size of result */
     result_size = size_DEFAULT;
-    if(result_type != E &&	/* Error type gets DEFAULT size */
-       opclass != tok_relop) {	/* Result of compare gets DEFAULT size */
+    if(result_type != E ) {	/* Error type gets DEFAULT size */
 
       if(opclass == tok_concat) {	/* string//string yields sum of lengths */
 	if(size1 == size_UNKNOWN || size2 == size_UNKNOWN)
@@ -405,9 +410,9 @@ if(debug_latest)
 	if(tc1 == tc2) {/* same type category */
 				/* Intrins args: size promotion illegal */
 	  if(INTRINS_ARGS && ls1 != ls2) {
-	    syntax_error(op->line_num,op->col_num,
+	      syntax_error(op->line_num,op->col_num,
 			 "precision mismatch in intrinsic argument list:");
-	    report_mismatch(term1,op,term2);
+	      report_mismatch(term1,op,term2);
 	  }
 				/* Give -port warning if e.g. plain I+I*2
 				   (variables only) */
@@ -482,11 +487,27 @@ if(debug_latest)
 		    "promotion may not give desired precision:");
 	    report_mismatch(term1,op,term2);
 	  }
+
+	  /* For comparisons we have R==R, C==C, or R==C. */
+	  if( opclass == tok_relop &&
+	      (type_category[t1] == type_category[t2]? ls2 != ls1:ls2 != 2*ls1) ) {
+	    warning(op->line_num,op->col_num,
+		    "comparison mixes terms of different precision:");
+	    report_mismatch(term1,op,term2);
+	  }
 	}
 
       }/*end if(non-DEFAULT sizes)*/
 
-    }/*end if(result_size != E)*/
+	/* Result of compare gets DEFAULT size.  However, we want to
+	   go thru the above code for relops in case of type/size
+	   mismatches that should be reported.  Here for relops we
+	   replace any result_size that got set above by DEFAULT size.
+	*/
+      if( opclass == tok_relop) {
+	   result_size = size_DEFAULT;
+      }
+    }/*end if(result_type != E)*/
 
 #ifdef DEBUG_EXPRTYPE
 if(debug_latest) {
@@ -603,8 +624,10 @@ is_true(EVALUATED_EXPR,result->TOK_flags));
 	    case '-': c = a-b; break;
 	    case '*': c = a*b; break;
 	    case '/': if(b == 0) {
-			syntax_error(term2->line_num,term2->col_num,
+		        if(misc_warn) {
+			  syntax_error(term2->line_num,term2->col_num,
 				"division by zero attempted");
+			}
 			c = 0;
 		      }
 		      else {
@@ -663,10 +686,12 @@ unexpr_type(op,term1,result)
 	result_type;
 
     if( ! is_computational_type(type1) ) {
+      if( misc_warn ) {
 		syntax_error(term1->line_num,term1->col_num,
 			"numeric quantity expected:");
 		report_type(term1);
-		result_type = E;
+      }
+      result_type = E;
     }
     else {
 	switch(opclass) {
@@ -698,11 +723,13 @@ unexpr_type(op,term1,result)
 
 	if( type1 != E ) {
 	    if( result_type == E) {
-		syntax_error(op->line_num,op->col_num,
+	      if( misc_warn ) {
+		  syntax_error(op->line_num,op->col_num,
 			"expression incompatible with operator:");
 		msg_tail(op->src_text);
 		msg_tail("used with");
 		report_type(term1);
+	      }
 	    }
 	    else if(result_type >= W) {
 	      if(f77_mixed_expr || f90_mixed_expr) {
@@ -769,16 +796,20 @@ assignment_stmt_type(term1,equals,term2)
 	result_type;
 
     if( ! is_computational_type(type1) ) {
+      if( misc_warn ) {
 		syntax_error(term1->line_num,term1->col_num,
 			"numeric or character quantity expected:");
 		report_type(term1);
-		result_type = E;
+      }
+      result_type = E;
     }
     else if( ! is_computational_type(type2) ) {
+      if( misc_warn ) {
 		syntax_error(term2->line_num,term2->col_num,
 			"numeric or character quantity expected:");
 		report_type(term2);
-		result_type = E;
+      }
+      result_type = E;
     }
     else {
 	result_type = (unsigned)assignment_type[type1][type2];
@@ -786,11 +817,13 @@ assignment_stmt_type(term1,equals,term2)
 
 	if( (type1 != E && type2 != E) ) {
 	    if( result_type == E) {
+	      if( misc_warn ) {
 		syntax_error(equals->line_num,equals->col_num,
 			"type mismatch:");
 		report_type(term2);
 		msg_tail("assigned to");
 		report_type(term1);
+	      }
 	    }
 	    else {
 	      if(result_type >= W) {		/* W result */
@@ -963,6 +996,29 @@ if(debug_latest) {
     use_lvalue(term1);
 }
 
+void
+check_initializer_type(Token *assignee_list, Token *equals, Token *expr_list)
+{
+    Token *t;
+    if( expr_list->next_token == (Token*)NULL ) {
+	t = expr_list;		/* simple token, not a list */
+    }
+    else {
+				/* token lists are built in reverse, so
+				   restore to order in source statement */
+	t = expr_list->next_token = reverse_tokenlist(expr_list->next_token);
+    }
+
+				/* Go thru list, checking match.
+				   At this time, assignee can only be a single
+				   variable
+				 */
+    while( t!=NULL ) {
+	assignment_stmt_type(assignee_list,t,t);
+	t = t->next_token;
+    }
+}
+
 	/* Make an expression-token for a function invocation */
 
 void
