@@ -1,4 +1,4 @@
-/* $Id: forlex.c,v 1.41 2001/11/03 00:55:37 moniot Exp $
+/* $Id: forlex.c,v 1.46 2004/12/31 17:22:16 moniot Exp $
 
 	Tokenizing routines for Fortran program checker.
 */
@@ -52,6 +52,15 @@ as the "MIT License."
 #include "tokdefs.h"
 #include "forlex.h"
 
+/* toascii() is widely supported, but in case it isn't, define it here.
+   We need it mainly in order to avoid bounds violation in legal_chars array.
+   On EBCDIC systems, toascii() should convert from an EBCDIC code
+   to the ASCII code for the same character, because the legal_chars array is
+   ordered according to the ASCII collating sequence.
+ */
+#ifndef toascii
+#define toascii(C) ((C) & 0177)
+#endif
 
 extern int complex_const_allowed,    /* shared flags operated by fortran.y */
 	   inside_format,
@@ -103,12 +112,13 @@ PRIVATE int closeup_saw_whitespace;
 #define islegal(C) ( ((C) == EOF) || ((C) == EOS) || \
 	( (col_num >= 6 || free_form || isdigit(C)) && \
 	 (toascii((int)(C)) >= toascii(' ') && \
-	  toascii((int)(C)) <= toascii('z') && \
 	  legal_chars[toascii((int)(C))-toascii(' ')] == (C))) )
 		/* Array has x where ASCII character is not valid.
 		   This defn is not exactly standard f77, since it includes
 		   supported extensions: $ in format,
 		   <> in variable formats, and " in strings.
+		   Since strlen of array is 96, indexing by [toascii(c)-32]
+		   is always in bounds.
 		 */
 PRIVATE char f77_legal_chars[]=
 " x\"x$xx'()*+,-./0123456789:x<=>xx\
@@ -166,7 +176,7 @@ make_legal_char(s)
   while( *s != '\0' ) {
     i = toascii((int)(*s));
     if(i >= toascii(' ') && i <= toascii('~')) {
-      legal_chars[i-' '] = *s;
+      legal_chars[i-toascii(' ')] = *s;
     }
     s++;
   }
@@ -229,6 +239,27 @@ PROTO(PRIVATE void get_simple_punctuation,( Token *token ));
 
 PROTO(PRIVATE int f90_relop, ( Token *token, int *multichar ) );
 
+/* Define zero_struct to set a struct to zero. It works around the differing availability
+   of memset and bzero.   This is used to initialize a token. 
+ */
+#if HAVE_MEMSET
+#define zero_struct(sptr,struct_size)    (void)memset((sptr),0,struct_size)
+#else
+#if HAVE_BZERO
+#define zero_struct(sptr,struct_size)    bzero((char *)(sptr),struct_size)
+#else
+PROTO(PRIVATE void zero_struct, (void *sptr, int struct_size) );
+PRIVATE void
+zero_struct(void *sptr, int struct_size)
+{
+     int i;
+     for(i=0; i<struct_size; i++) {
+	  ((char *)sptr)[i] = 0;
+     }
+}
+#endif
+#endif
+
 		/*  Gets next token for Yacc.  Return value is token.class,
 		 *  and a copy of the token is stored in yylval.
 		 */
@@ -239,15 +270,8 @@ yylex(VOID)
     extern int in_attrbased_typedecl; /* shared with fortran.y */
 
 		/* Initialize token fields to scratch. */
-#if HAVE_MEMSET
-    (void)memset(&token,0,sizeof(token));
-#else
-#if HAVE_BZERO
-    bzero((char *)&token,sizeof(token));
-#else
-    token = (Token){0};	/* If neither one, hope compiler likes this */
-#endif
-#endif
+    zero_struct(&token,sizeof(token));
+
     src_text_len = 0;
 
     if(curr_char == EOF) {
@@ -314,6 +338,7 @@ yylex(VOID)
     if(token.tclass == EOS) {
 	implicit_flag=FALSE;	/* in case of errors, reset flags */
 	implicit_letter_flag = FALSE;
+	WHILE_expected = FALSE;
     }
 
 
@@ -410,9 +435,12 @@ get_cpp_directive(VOID)
 		   # 1 "" since that means cpp was working with stdin,
 		   probably from ftnpp.  Likewise ignore # 1 "stdin"
 		   which is a variant form, e.g. from fpp.
+		   Nowadays cpp may indicate stdin by "<stdin>" or suchlike,
+		   which we also ignore.
 		*/
     if( cpp_start_of_file ) {
       if( next_filename[0] != '\0' &&
+	  next_filename[0] != '<' &&
 	  strcmp(next_filename,"stdin") != 0 ) {
 	top_filename = next_filename;
 	current_filename = next_filename;
@@ -439,16 +467,17 @@ get_cpp_directive(VOID)
 
     
   }
-  do {			/* Skip to end of directive.  It will become an EOS */
-    advance();
-  } while( curr_char != EOS);
-
   if(f77_unix_cpp || f90_unix_cpp || !cpp_handled) {
     nonstandard(line_num,col_num,f90_unix_cpp,0);
     msg_tail(": preprocessor directive");
     if(!cpp_handled)
       msg_tail("(not processed)");
   }
+
+  do {			/* Skip to end of directive.  It will become an EOS */
+    advance();
+  } while( curr_char != EOS);
+
 }/*get_cpp_directive*/
 #endif
 
@@ -1319,6 +1348,10 @@ if(sign < 0.0) (void)fprintf(list_fd," -");
 }
 #endif
 	closeup_saw_whitespace = FALSE; 
+		/* Initialize imag_part token fields. */
+	zero_struct(&imag_part,sizeof(imag_part));
+	imag_part.line_num = line_num;
+	imag_part.col_num = col_num;
 	get_number(&imag_part);
 	imag_dble_size = (imag_part.tclass == tok_dp_const);
 
@@ -1512,6 +1545,13 @@ get_punctuation(token)
 		multichar = TRUE;
 		advance();
 		src_text_buf[src_text_len++] = curr_char;
+	}
+                                /* recognize F90 pointer assignment => */
+        else if(curr_char == '=' && next_char == '>') {
+                token->tclass = tok_pointer_assignment;
+                multichar = TRUE;
+                advance();
+		src_text_buf[src_text_len++] = curr_char;
 	}
 				/* recognize F90 rel-ops here */
 	else if( f90_relop(token,&multichar) ) {
@@ -1678,7 +1718,7 @@ get_string(token)       /* Gets string of form 'aaaa' */
 			   blanks to 72 columns.  Free form: line ends at EOL */
 		  if( ! free_form ) {
 		    COLNO_t col;
-		    for(col=last_col_num; col<(COLNO_t)max_stmt_col; col++) {
+		    for(col=last_col_num; col<max_stmt_col; col++) {
 
 		      if(src_text_len < MAX_SRC_TEXT)
 			src_text_buf[src_text_len++] = ' ';
@@ -1704,11 +1744,6 @@ get_string(token)       /* Gets string of form 'aaaa' */
 			  src_text_buf[src_text_len++] = curr_char;
 #endif
 
-				    /* Handle possible continuation */
-			if(next_char == EOL &&
-			   (free_form || col_num == (COLNO_t)max_stmt_col))
-			  advance();
-
 			last_col_num=col_num;
 			advance();
 
