@@ -1,4 +1,4 @@
-/* $Id: ftnchek.c,v 1.45 2003/03/20 22:14:02 moniot Exp $
+/* $Id: ftnchek.c,v 1.47 2004/12/29 19:27:25 moniot Exp $
 
 	Main program for Fortran Syntax Checker.
 */
@@ -51,6 +51,9 @@ as the "MIT License."
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_STRINGS_H
+#include <strings.h>	/* we use strncasecmp */
+#endif
 #include <ctype.h>
 #ifdef DEVELOPMENT             /* For maintaining the program */
 #define DEBUG_SIZES
@@ -345,7 +348,6 @@ do_action:
 
 		    }
 
-		    (void) fclose(input_fd);
 		}/*end processing file args*/
 	      }
 next_arg:
