@@ -1,4 +1,4 @@
-/* $Id: ftnchek.h,v 1.86 2003/03/20 22:13:17 moniot Exp $
+/* $Id: ftnchek.h,v 1.94 2005/02/06 21:36:37 moniot Exp $
 
 	Common definitions for Fortran Program Checker
 */
@@ -38,13 +38,13 @@ changes, from the XFree86 Project.
 */
 
 #define COPYRIGHT_DATE \
-"Copyright (C) 1994, 1997, 2000 by Robert K. Moniot"
+"Copyright (C) 1994, 1997, 2000, 2004 by Robert K. Moniot"
 #define COPYRIGHT_NOTICE \
 "This program is free software.  Permission is granted to\n\
 modify it and/or redistribute it, retaining this notice.\n\
 No guarantees accompany this software."
-#define VERSION_NUMBER		"FTNCHEK Version 3.2 November 2002"
-#define PATCHLEVEL		"Patch Level 2"
+#define VERSION_NUMBER		"FTNCHEK Version 3.3 November 2004"
+#define PATCHLEVEL		"Patch Level 1"
 #define PROJECT_VERSION		"P3" /* Project file format version number */
 
 #include "config.h"		/* Get system-specific information */
@@ -318,15 +318,6 @@ PROTO(VOID exit,(int));
 #define F95_ALL 0
 #endif
 
-			/* -mkhtml requires regex support.  This macro
-			   controls only whether the -mkhtml command-line
-			   option is made available.  The relevant variables
-			   and make_html function are still defined, but the
-			   latter (if somehow invoked) just apologizes.
-			 */
-#if HAVE_REGEX_H
-#define HTML_OPTION
-#endif
 
 			/* -portability options default to PORT_ALL.  Define
 			   STRICT_PORTABILITY to make -port=all the default.
@@ -900,6 +891,7 @@ OPT(int,f77_attrbased_typedecl,F77_ALL);
 OPT(int,f77_automatic_array,F77_ALL);
 OPT(int,f77_byte,F77_ALL);
 OPT(int,f77_case_construct,F77_ALL);
+OPT(int,f77_char_extension,F77_ALL);
 OPT(int,f77_cray_pointers,F77_ALL);
 OPT(int,f77_common_subprog_name,F77_ALL);
 OPT(int,f77_construct_name,F77_ALL);
@@ -927,6 +919,7 @@ OPT(int,f77_overlength,F77_ALL);
 OPT(int,f77_param_implicit_type,F77_ALL);
 OPT(int,f77_param_intrinsic,F77_ALL);
 OPT(int,f77_param_noparen,F77_ALL);
+OPT(int,f77_pointers,F77_ALL);
 OPT(int,f77_quad_constants,F77_ALL);
 OPT(int,f77_quotemarks,F77_ALL);
 OPT(int,f77_relops,F77_ALL);
@@ -1144,7 +1137,7 @@ OPT(int,debug_parser,FALSE);	/* grammar debug via DBG statements */
 OPT(int,debug_hashtab,FALSE);	/* hash table contents */
 OPT(int,debug_loc_symtab,FALSE); /* local symtab contents */
 OPT(int,show_resources,FALSE);	/* space avail and used */
-#ifdef DEBUG_FORLEX
+#if defined(DEBUG_FORLEX) || defined(DEBUG_IS_KEYWORD)
 OPT(int,debug_lexer,FALSE);	/* list of tokens as scanned */
 #endif
 
@@ -1289,14 +1282,16 @@ PROTO(void init_typesizes, ( void ));
 PROTO( char * add_ext,( char *s, const char *ext ));
 PROTO( int has_extension,( const char *name, const char *ext ));
 
-	/* in forlex.c */
+	/* in forlex.c and advance.c */
 PROTO(void make_legal_char_list, ( void ));
 PROTO(void finish_scan, ( void ));
 PROTO(int flush_line_out, ( LINENO_t n ));
+PROTO(int flush_end_stmt, ( LINENO_t n ));
 PROTO(void init_keyhashtab, ( void ));
 PROTO(void init_scan, ( void ));
 PROTO(void init_stream,( void ));
 
+
 	/* in include.c */
 PROTO(void open_include_file, ( char *fname, LINENO_t include_line_num ));
 PROTO(int pop_include_file,( void ));
