@@ -1,4 +1,4 @@
-/* $Id: intrins.c,v 1.3 2001/11/03 00:55:37 moniot Exp $
+/* $Id: intrins.c,v 1.4 2003/06/05 00:19:59 landrito Exp $
 
 	Handles datatyping of intrinsic functions.
 */
@@ -171,7 +171,14 @@ PRIVATE IntrinsInfo intrinsic[]={
 {"LGT", 	2,	STR,	type_LOGICAL,	I_F77|I_NOTARG},
 {"LLE", 	2,	STR,	type_LOGICAL,	I_F77|I_NOTARG},
 {"LLT", 	2,	STR,	type_LOGICAL,	I_F77|I_NOTARG},
-
+                /* associated intrinsic statement to check association of
+                 * pointers
+                 */
+{"ASSOCIATED",  I_1or2, I|R|D|C|Z|L|STR,type_LOGICAL, I_NONF77|I_NONPURE},
+                /* allocated intrinsic statement to check allocation of
+                 * pointers
+                 */
+{"ALLOCATED",   1,      I|R|D|C|Z|L|STR,type_LOGICAL, I_NONF77|I_NONPURE},
 		/* DOUBLE COMPLEX intrinsics are included regardless
 		   of -intrinsics option, since they are essential
 		   to support of this datatype.
