@@ -1,4 +1,4 @@
-/* $Id: keywords.c,v 1.11 2001/10/08 00:49:14 moniot Exp $
+/* $Id: keywords.c,v 1.13 2003/06/05 00:19:59 landrito Exp $
 
 	Routines to distinguish keywords from identifiers.
 
@@ -109,6 +109,8 @@ PRIVATE struct {
 	short split_pos;	/* where keyword may have space */
 } keywords[]={
 {"ACCEPT",	tok_ACCEPT,	IK | EK,			0},
+{"ALLOCATABLE", tok_ALLOCATABLE,        IK | NI | EK | NP,      0},
+{"ALLOCATE",	tok_ALLOCATE,	IK | MP | NI | EK,		0},
 {"ASSIGN",	tok_ASSIGN,	IK | NP | EK | NA,		0},
 {"BACKSPACE",	tok_BACKSPACE,	IK | EK,			0},
 {"BLOCKDATA",	tok_BLOCKDATA,	IK | EK | NP | NI,		5},
@@ -123,6 +125,7 @@ PRIVATE struct {
 {"CONTINUE",	tok_CONTINUE,	IK | NP | EK | NA,		0},
 {"CYCLE",	tok_CYCLE,	IK | NP | EK,			0},
 {"DATA",	tok_DATA,	IK | NI | EK,			0},
+{"DEALLOCATE",	tok_DEALLOCATE,	IK | MP | NI | EK,		0},
 {"DIMENSION",	tok_DIMENSION,	IK | NP | NI | EK,		0},
 {"DO",		tok_DO,		IK | NP | NI | EK | CN,		0},
 {"DOUBLECOMPLEX",tok_DOUBLECOMPLEX,	IK | NI | EK | TY,	6},
@@ -157,11 +160,12 @@ PRIVATE struct {
 {"LOGICAL",	tok_LOGICAL,	IK | NI | EK | TY | TK,		0},
 {"NAMELIST",	tok_NAMELIST,	IK | NP | NI | EK,		0},
 {"NONE",	tok_NONE,	IK | NI | EK | TY | NA,		0},
+{"NULLIFY",	tok_NULLIFY,	IK | MP | NI | EK,		0},
 {"OPEN",	tok_OPEN,	IK | EK | MP | NA,		0},
 {"PARAMETER",	tok_PARAMETER,	IK | NI | EK | MP | NA,		0},
 {"PAUSE",	tok_PAUSE,	IK | NP | EK,			0},
 #ifdef ALLOW_CRAY_POINTERS
-{"POINTER",     tok_POINTER,    IK | MP | NI | EK | NA,		0},
+{"POINTER",     tok_POINTER,    IK | NI | EK,			0},
 #endif
 {"PRINT",	tok_PRINT,	IK | EK,			0},
 {"PROGRAM",	tok_PROGRAM,	IK | NP | NI | EK,		0},
@@ -173,6 +177,7 @@ PRIVATE struct {
 {"SELECTCASE",	tok_SELECTCASE,	IK | MP | NI | EK | NA | CN | MB,6},
 {"STOP",	tok_STOP,	IK | NP | EK,			0},
 {"SUBROUTINE",	tok_SUBROUTINE,	IK | NP | NI | EK,		0},
+{"TARGET",      tok_TARGET,     IK | NI | EK | NP,              0},
 {"THEN",	tok_THEN,	IK | NP | EK,			0},
 {"TO",		tok_TO,		NI | EK,			0},
 {"TYPE",	tok_TYPE,	IK | EK,			0},
@@ -545,10 +550,15 @@ is_keyword(i)
 			/* Dispose of keywords which can only occur in initial
 			   part of statement, if found elsewhere. One exception
 			   is something with a construct-name tacked on in
-			   front of it, which is a cinch.  Only DO is
-			   recognized this way so far.
+			   front of it, which is a cinch.
 			*/
-    ans = (construct_name_seen && MATCH(CN));
+    if( (ans = (construct_name_seen && MATCH(CN))) ) {
+      if( putative_keyword_class == tok_DO &&
+	  looking_at_keywd(tok_WHILE) ) {
+	WHILE_expected = TRUE;
+      }
+    }
+
   }
 
 #if 0 /* This does not work: curr_stmt_class not cleared beforehand */
@@ -593,7 +603,8 @@ is_keyword(i)
     ans = FALSE; /* otherwise it will be handled correctly by looking_at */
   }
   else if(putative_keyword_class == tok_WHILE) { /* A non-IK case */
-    ans = WHILE_expected; /* Only occurs in DO label [,] WHILE */
+	 /* Only occurs in DO label [,] WHILE and constructname: DO WHILE  */
+    ans = WHILE_expected;
     WHILE_expected = FALSE;
   }
 		/* Remaining cases are IK in initial part */
