@@ -1,7 +1,8 @@
-/*   $Id: labels.c,v 1.19 2002/11/20 23:13:47 moniot Rel $
+/*   $Id: labels.c,v 1.22 2004/12/29 19:40:21 moniot Exp $
  *   Handles defs and refs of statement labels
  *
  *   Written by Heba Elsayed
+ *   Modified to use an open hash backend by Robert Landrito
  */
 
 /*
@@ -44,6 +45,7 @@ as the "MIT License."
 #include "plsymtab.h"
 
 #define TOP_FILE -1               /* not an include file */                  
+#define HASHSIZE 50               /* hash table size */
 
 
 /* Linked-list: describe how references are used */
@@ -66,7 +68,7 @@ typedef struct lab_ref_list
 
 /* A label */
 
-typedef struct {
+typedef struct label_node {
     LABEL_t lab;                  /* label numeric value */
     short file_def;               /* file where defined; = TOP_FILE if
 				     not an include file, else = 
@@ -81,12 +83,15 @@ typedef struct {
     unsigned referenced:1;        /* TRUE if label is referenced */
     unsigned do_label:1;	  /* TRUE if label used in DO statement */
     Lab_ref_list *how_used;       /* list describing each ref to label */
+    struct label_node *next;      /* pointer to next label in linked list */
 } Lab;
 
 
 /* >>>>> function prototypes <<<<< */
 
-PRIVATE void insert_lab(int *i, LABEL_t label);
+PRIVATE unsigned int hash_label(LABEL_t label);
+
+PRIVATE int insert_lab(LABEL_t label);
 
 PRIVATE void ref_error_check(char *mod_name, int type1, int type2, int type3,
 			     Lab *label); 
@@ -136,29 +141,130 @@ PRIVATE unsigned any_lab_defined;
 PRIVATE unsigned any_lab_referenced;        
    /* used to check if any label has been referenced */
 
+PRIVATE Lab *hash_table[HASHSIZE];
+   /* array of pointers to label linked list for use in hashing scheme */
+
+PRIVATE unsigned int last_hash_index;
+   /* last hash index searched */
+
+/*
+ * hash: get a hash index
+ */
+PRIVATE unsigned int hash_label(LABEL_t label) {
+    /* numbers used in the following random number generator obtained from
+       Numerical Recipes in C: The Art of Scientific Computing */
+    return ( (1861 * label + 49297) % 233280 );
+}
+
+
+/*
+ * init_labtable: initialize space for labels in labtable
+ */
+void init_labtable(void) {
+    int i;
+    num_labels = ref_index = last_hash_index = 0;
+    any_lab_defined = any_lab_referenced = FALSE;
+
+    if ( labtable_size == 0 ) {
+        labtable_size = LABTABLE_SIZE;
+
+        if ( NULL == (labtable = (Lab *) malloc(sizeof(Lab) * labtable_size)) )
+            oops_message(OOPS_FATAL, NO_LINE_NUM, NO_COL_NUM,
+                    "Cannot alloc space for label table");
+    } /* end (outer) if */
+
+    /* nullify hash table */
+    for (i = 0; i < HASHSIZE; i++)
+        hash_table[i] = NULL;
+
+    return;
+}
+
+/*
+ * grow_labtable: double the size of the hashtable
+ */
+PRIVATE void grow_labtable(void) {
+    Lab *old_labtable = labtable;
+    int i;
+
+    labtable_size *= 2;
+    if ( NULL == ( labtable =
+                (Lab *) realloc(labtable, sizeof(Lab) * labtable_size) ) )
+        oops_message(OOPS_FATAL, NO_LINE_NUM, NO_COL_NUM,
+                "Cannot realloc space for label tabel");
+
+#ifdef DEBUG_LABELS
+    fprintf(stderr, "DEBUG: resizing labtable to %d...", labtable_size);
+#endif
+
+    /* fix the pointers in hash_table */
+    for (i = 0; i < HASHSIZE; i++)
+        if (hash_table[i] != NULL)
+            hash_table[i] = labtable + (hash_table[i] - old_labtable);
+
+    /* fix pointers in linked lists */
+    for (i = 0; i < num_labels; i++)
+        if (labtable[i].next != NULL)
+            labtable[i].next = labtable + (labtable[i].next - old_labtable);
+
+#ifdef DEBUG_LABELS
+    fprintf(stderr, "DONE\n");
+#endif
+
+    return;
+}
+
+/*
+ * insert_lab: inserts new label into labtable
+ */
+PRIVATE int insert_lab(LABEL_t label) {
+
+    /* check if we have any space */
+    if ( num_labels >= labtable_size )
+        grow_labtable();
+
+    /* place label in linked list pointed to in hash_table */
+    labtable[num_labels].next = hash_table[last_hash_index];
+    hash_table[last_hash_index] = &labtable[num_labels];
+
+    /* set up the label */
+    labtable[num_labels].how_used = NULL;
+
+    labtable[num_labels].defined = FALSE;
+    labtable[num_labels].referenced = FALSE;
+    labtable[num_labels].do_label = FALSE;
+
+    labtable[num_labels].file_def = TOP_FILE;
+    labtable[num_labels].line_def = NO_LINE_NUM;
+    labtable[num_labels].stmt_type = LAB_NO_TYPE;
+
+    labtable[num_labels].lab = label;
+
+    ++num_labels;
+
+    return (num_labels - 1);
+}
 
 /*
  * sort_labtable
+ * WARNING: THIS RENDERS THE HASH TABLE USELESS
+ *          DO *NOT REFERENCE THE HASH TABLE AFTER
+ *          CALLING THIS FUNCTION, UNLESS init_labtable()
+ *          IS FIRST CALLED.
  */
-
 void sort_labtable(void)
 {
-    int i, j, k;
+    int j, k;
     int count = 0;  /* number of unsorted pairs */
 
-    /* squeeze out blank slots */
 
-    for (i = j = 0; j < labtable_size; j++) 
+    for (j = 0; j < num_labels; j++) 
     {
-	if (labtable[j].lab != NO_LABEL) 
-	{  		
 	    /* Put ref lines into ascending order */
 
 	    labtable[j].how_used = 
 		reverse_ref_list(labtable[j].how_used);
 
-	    labtable[i++] = labtable[j];
-	}
     }
                      /* assert: i == num_labels */
     
@@ -452,15 +558,21 @@ void print_labels(void)
  */
 
 PRIVATE int find_label(LABEL_t label)
-{
-    int h = label % labtable_size;     /* index of label */
+{  
+    Lab *label_ptr;
 
-    while ((labtable[h].lab != NO_LABEL) && (labtable[h].lab != label))
-    {
-	h = (h + 1) % labtable_size;
-    }
+    /* index value of linked list where "label" is a part of */
+    last_hash_index = (hash_label(label) >> 9) % HASHSIZE;
+    label_ptr = hash_table[last_hash_index];
 
-    return h;
+    while ( label_ptr != NULL && label_ptr->lab != label )
+        label_ptr = label_ptr->next;
+
+    if ( label_ptr == NULL )    /* not found! */
+        return NO_LABEL;
+
+    /* return the index of *labtable* */
+    return (label_ptr - labtable);
 }
 
 /*
@@ -475,9 +587,9 @@ int def_label(Token *t, int type)
     
     int i = find_label(label);        /* label's index */
     
-    if (labtable[i].lab == NO_LABEL)  /* label never defined/used */
+    if (i == NO_LABEL)  /* label never defined/used */
     {
-	insert_lab(&i, label);
+	i = insert_lab(label);
     }
     
     any_lab_defined = TRUE;
@@ -511,7 +623,7 @@ void def_do_label(Token *t)
     int i = find_label(label);        /* label's index */ 
 
 
-    if (labtable[i].lab == NO_LABEL)  /* label has to be defined by now */
+    if (i == NO_LABEL)  /* label has to be defined by now */
     {
 	oops_message(OOPS_FATAL,t->line_num,t->col_num,
 		     "def_do_label called for undefined label");
@@ -532,9 +644,9 @@ void ref_label(Token *t, int type)
     int i = find_label(label);        /* label's index */ 
 
 
-    if (labtable[i].lab == NO_LABEL)  /* label never defined/used */
+    if (i == NO_LABEL)  /* label never defined/used */
     {
-	insert_lab(&i, label);
+	i = insert_lab(label);
     }
 
     any_lab_referenced = TRUE;
@@ -596,124 +708,6 @@ void ref_label(Token *t, int type)
 }
 
 
-/*
- * insert_lab: inserts new label into labtable
- *
- */
-
-PRIVATE void insert_lab(int *labtab_index, LABEL_t label)
-{
-    int i;
-    if (num_labels > labtable_size/2)  /* labtable is half full: make bigger */
-    {
-      int old_labtable_size = labtable_size;
-      Lab *old_labtable = labtable;
-      labtable_size = labtable_size*2 + 1; /* allocate a table twice as big */
-#ifdef DEBUG_LABELS
-      if (debug_latest)
-      {
-	(void)fprintf(list_fd,"\nResizing label table from %d to %d",
-		old_labtable_size,labtable_size);
-	(void)fprintf(list_fd,"\nOld label table:");
-	for (i = 0; i < old_labtable_size; i++)
-	{
-	  if (old_labtable[i].lab != NO_LABEL)
-	  {
-	    (void)fprintf(list_fd,"\n%3d %3d <%d> def %d",
-		    i,old_labtable[i].lab%old_labtable_size,
-		    old_labtable[i].lab,
-		    old_labtable[i].line_def);
-	  }
-	}
-      }
-#endif
-      if((labtable=(Lab *)malloc(labtable_size*sizeof(Lab))) == (Lab *)NULL)
-      {
-	oops_message(OOPS_FATAL, NO_LINE_NUM, NO_COL_NUM,
-		     "Cannot realloc space for label table");
-      }
-				/* Initialize the new label table */
-      for (i = 0; i < labtable_size; i++)
-      {
-	labtable[i].lab = NO_LABEL;
-      }
-				/* Copy the old table to the new, resolving
-				   clashes which may no longer clash.
-				 */
-      for (i = 0; i < old_labtable_size; i++)
-      {
-	if (old_labtable[i].lab != NO_LABEL)
-	{
-	  int h = find_label(old_labtable[i].lab);
-	  if (labtable[h].lab != NO_LABEL) {
-	      oops_message(OOPS_FATAL, NO_LINE_NUM, NO_COL_NUM,
-			   "assertion failed: labtable[h].lab != NO_LABEL");
-	  }
-	  labtable[h] = old_labtable[i];
-	}
-      }
-      free(old_labtable);
-
-#ifdef DEBUG_LABELS
-      if (debug_latest)
-      {
-	(void)fprintf(list_fd,"\nNew label table:");
-	for (i = 0; i < labtable_size; i++)
-	{
-	  if (labtable[i].lab != NO_LABEL)
-	  {
-	    (void)fprintf(list_fd,"\n%3d %3d <%d> def %d",
-		    i,labtable[i].lab%labtable_size,
-		    labtable[i].lab,
-		    labtable[i].line_def);
-	  }
-	}
-      }
-#endif
-				/* Find the label in the new table */
-      *labtab_index = find_label(label);
-    }
-
-    i = *labtab_index;
-
-	++num_labels;
-	labtable[i].how_used = NULL;
-	
-	labtable[i].defined = FALSE;
-	labtable[i].referenced = FALSE;
-	labtable[i].do_label = FALSE;
-	
-	labtable[i].file_def = TOP_FILE;
-    	labtable[i].line_def = NO_LINE_NUM;
-	labtable[i].stmt_type = LAB_NO_TYPE;  
-	
-	labtable[i].lab = label;   
-
-}
-
-/*
- * init_labtable: initialize labtable
- */
-
-void init_labtable(void)
-{
-    int i;
-    num_labels = ref_index = 0;
-    any_lab_defined = any_lab_referenced = FALSE;
-
-    if( labtable_size == 0 ) {
-      labtable_size = LABTABLE_SIZE;
-      if((labtable=(Lab *)malloc(labtable_size*sizeof(Lab))) == (Lab *)NULL) {
-	oops_message(OOPS_FATAL, NO_LINE_NUM, NO_COL_NUM,
-		     "Cannot malloc space for label table");
-      }
-    }
-    for (i = 0; i < labtable_size; i++)
-    {
-	labtable[i].lab = NO_LABEL;
-    }
-}
- 
 /* 
  * check_labels: report problems with label usage
  */
