@@ -1,4 +1,4 @@
-/* $Id: options.c,v 1.36 2003/03/20 22:15:42 moniot Exp $
+/* $Id: options.c,v 1.42 2004/12/29 19:27:25 moniot Exp $
 
 	Definitions of command-line options and routines to set them.
 
@@ -37,6 +37,9 @@ as the "MIT License."
 
 #include <stdio.h>
 #include <string.h>
+#if HAVE_STRINGS_H
+#include <strings.h>	/* we use strncasecmp */
+#endif
 #include <ctype.h>
 #include "ftnchek.h"
 #include "options.h"	/* Prototypes of external routines defined below */
@@ -254,7 +257,7 @@ PRIVATE struct {
 		 "debug printout of hashtable",IS_A_CHECK},
 	{"local",	&debug_loc_symtab,
 		 "debug local symtab info",IS_A_CHECK},
-#ifdef DEBUG_FORLEX
+#if defined(DEBUG_FORLEX) || defined(DEBUG_IS_KEYWORD)
 	{"tokens",	&debug_lexer,
 		 "debug printout in lexer",IS_A_CHECK},
 #endif
@@ -445,6 +448,8 @@ PRIVATE WarnOptionList
 				"BYTE data type"},
   {"case-construct",	&f77_case_construct,
 				"CASE construct"},
+  {"character",		&f77_char_extension,
+				"Character variable defined length <= 0"},
   {"common-subprog-name",&f77_common_subprog_name,
 				"Common block & subprog with same name"},
   {"construct-name",	&f77_construct_name,
@@ -453,6 +458,8 @@ PRIVATE WarnOptionList
 				"More than 19 continuation lines"},
   {"cpp",		&f77_unix_cpp,
 				"Unix C preprocessor directives"},
+  {"cray-pointer",	&f77_cray_pointers,
+				"Cray pointer syntax"},
   {"cycle-exit",	&f77_cycle_exit,
 				"CYCLE or EXIT statement"},
   {"d-comment",		&f77_d_comment,
@@ -503,8 +510,8 @@ PRIVATE WarnOptionList
 				"Intrinsics and **real in PARAMETER defns"},
   {"param-noparen",	&f77_param_noparen,
 				"PARAMETER statement without parens"},
-  {"pointer",		&f77_cray_pointers,
-				"Cray pointer syntax"},
+  {"pointer",		&f77_pointers,
+				"Fortran 90 pointer syntax"},
   {"quad-constant",	&f77_quad_constants,
 				"Quad precision constants like 1.23Q4"},
   {"quotemark",		&f77_quotemarks,
@@ -548,6 +555,8 @@ PRIVATE WarnOptionList
 				"Too many or empty continuation lines"},
   {"cpp",		&f90_unix_cpp,
 				"Unix C preprocessor directives"},
+  {"cray-pointer",	&f90_cray_pointers,
+				"Cray pointer syntax"},
   {"d-comment",		&f90_d_comment,
 				"Debug comments starting with D"},
   {"dec-tab"	,	&f90_dec_tabs,
@@ -577,8 +586,6 @@ PRIVATE WarnOptionList
 				"implicit typing of PARAMETERs"},
   {"param-noparen",	&f90_param_noparen,
 				"PARAMETER statement without parens"},
-  {"pointer",		&f90_cray_pointers,
-				"Cray pointer syntax"},
   {"quad-constant",	&f90_quad_constants,
 				"Quad precision constants like 1.23Q4"},
   {"statement-order",	&f90_stmt_order,
@@ -1034,12 +1041,10 @@ PRIVATE StrsettingList strsetting[]={
   {"makedcls",  &makedcl_warn_list, "declarations", "none", NOT_A_CHECK,
      makedcl_warn_option, makedcl_numeric_option,
     "make type declaration statements"},
-#ifdef HTML_OPTION
 				/* mkhtml similar to makedcls */
   {"mkhtml",	&mkhtml_warn_list, "documents","none",NOT_A_CHECK,
      mkhtml_warn_option, mkhtml_numeric_option,
     "create html documents"},
-#endif
   {"output",	&out_fname,	(char *)NULL, (char *)NULL, NOT_A_CHECK,
      (WarnOptionList *)NULL, NULL,
      "output file name"},
@@ -1383,8 +1388,9 @@ set_option(s,where)
 	    if( strncmp(s+orig_offset,setting[i].name,OPT_MATCH_LEN) == 0) {
 		char *numstr;
 
-		numstr = s + offset + OPT_MATCH_LEN;
-		while(++numstr, ! END_OF_OPT(*numstr) )
+		
+		for( numstr = s + offset + OPT_MATCH_LEN;
+		     ! END_OF_OPT(*numstr); numstr++ )
 		{
 		    if((*numstr == '=') || (*numstr == ':'))
 		    {			/* Find the assignment operator */
