@@ -1,4 +1,4 @@
-/* $Id: plsymtab.c,v 1.32 2001/11/03 00:51:21 moniot Exp $
+/* $Id: plsymtab.c,v 1.33 2003/03/26 01:16:28 moniot Exp $
 
 		Routines associated with printing of local symbol table info
 
@@ -716,11 +716,11 @@ debug_symtabs(VOID) 	/* Debugging output: hashtable and symbol tables */
 		      glob_symtab[i].size
 		);
 	    (void)fprintf(list_fd,
- ": use%d set%d asg%d ubs%d lib%d ent%d inf%d vis%d smw%d def%d incl%d ext%d ",
+ ": use%d set%d asg%d rec%d lib%d ent%d inf%d vis%d smw%d def%d incl%d ext%d ",
 		glob_symtab[i].used_flag,
 		glob_symtab[i].set_flag,
 		glob_symtab[i].assigned_flag,
-		glob_symtab[i].used_before_set,
+		glob_symtab[i].recursive,
 		glob_symtab[i].library_module,
 		glob_symtab[i].internal_entry,
 		glob_symtab[i].invoked_as_func,
