@@ -1,4 +1,4 @@
-/* $Id: prlists.c,v 1.5 2001/08/26 16:24:38 moniot Exp $
+/* $Id: prlists.c,v 1.10 2003/03/30 18:35:14 moniot Exp $
 
   Definition of process_lists() and associated routines that transfer
   argument and common-block lists from local to global symbol table
@@ -200,13 +200,14 @@ else {
 			     loc_symtab[i].info.arglist = a;
 			     head_ptr = head_ptr->next;
 		          }/* end while (head_ptr != NULL) */
-
+			  /* used_flag=1 does not imply call of the routine
+			     itself unless it is recursive.  (Recursive procs
+			     are not implemented yet.)  So do not copy it
+			     from local to global symbol table.
+			   */
 			  if(loc_symtab[i].set_flag)
 			         gsymt->set_flag =
 				   gsymt->set_this_file = TRUE;
-			  if(loc_symtab[i].used_flag)
-			         gsymt->used_flag =
-				   gsymt->used_this_file = TRUE;
 			  if(loc_symtab[i].library_module)
 				 gsymt->library_module = TRUE;
 			  gsymt->defined = TRUE;
