@@ -1,4 +1,4 @@
-/* $Id: prlocsym.c,v 1.17 2001/09/03 13:02:20 moniot Exp $
+/* $Id: prlocsym.c,v 1.22 2003/03/17 17:47:56 moniot Exp $
 
 	Prints local symbol table, calling local checking routines along
 	the way.
@@ -49,6 +49,11 @@ as the "MIT License."
 #include "plsymtab.h"
 #include "loccheck.h"
 
+PROTO(PRIVATE void print_io_unit_usages,(VOID));
+
+PROTO(PRIVATE void sort_io_unit_usages,(VOID));
+PROTO(PRIVATE int cmp_io_units, ( IO_Unit_Info *u1, IO_Unit_Info *u2 ));
+
 void
 print_loc_symbols(VOID)
 {
@@ -536,6 +541,11 @@ print_loc_symbols(VOID)
 
     }/*if(port_mixed_size/type)*/
 
+					/*  print table of I/O usages */
+    if( do_symtab && num_io_unit_usages > 0 ) {
+      print_io_unit_usages();
+    }
+
 					/* print table of stmt labels */
 
     if(misc_warn || usage_label_undefined || usage_label_unused
@@ -553,14 +563,182 @@ print_loc_symbols(VOID)
     }
 
     make_declarations(sym_list,mod_name);
+    
+   /* Recreate a FULL list of local symbols */
+   {
+   int i, n;
+   for(i=0,n=0;i<loc_symtab_top;i++)
+      {
+      sym_list[n++] = &loc_symtab[i];
+      }
+   }
+   
+   /* Call make_html to create an individual html doc for this FORTRAN module */
+   make_html(sym_list,mod_name, module);
+   
 }/* print_loc_symbols */
 
+/* Routine to print list of I/O unit usages.  It first sorts the list
+ * according to unit number/id and operation.  Following the
+ * identification of the I/O unit and operation, it lists all the line
+ * numbers where that occurs.
+ */
 
+PRIVATE void
+print_io_unit_usages(VOID)
+{
+      int i;
+			/* Arrays of names of access and formatting modes.
+			   Entries for default are left blank.
+			 */
+      static char* IO_access[]={"","DIR","SEQ"};
+      static char* IO_form[] ={"","UNF","FMTD"};
 
+			/* variables for remembering what was printed */
+      int old_id, old_no, old_op, old_acc, old_form;
+      int cols;
 
+      sort_io_unit_usages();	/* put list into sorted order */
 
+				/* print the heading of table */
+      fprintf(list_fd,"\nI/O Operations:\n");
+      fprintf(list_fd,"\n     Unit ID Unit No. Access Form Operation   Line");
+      for(i=0; i<num_io_unit_usages; i++) {
+	const char *unit_name;
+	int id = io_unit_info[i].unit_id;
+	int no = io_unit_info[i].unit_no;
+	int op = io_unit_info[i].io_operation;
+			/*  print info if it has changed */
+	if( i == 0 || id != old_id || no != old_no || op != old_op ||
+	    io_unit_info[i].io_access != old_acc ||
+	    io_unit_info[i].io_form != old_form ) {
+				/* print unit number or blank if unknown */
+				/* print unit name or blank if unknown */
+	   if( id < 0) {
+			/* handle cases of unknown and default */
+	      unit_name = (id == IO_UNIT_DEFAULT)? "*": "";
+	   }
+	   else {
+			/* handle cases where it is an identifier, unit_id=hashnum */
+	      unit_name = hashtab[id].name;
+	   }
+	   fprintf(list_fd,"\n%12s%1s",unit_name,
+				/* if id is a parameter, print name=value */
+		   (id >= 0 && no >= 0)? "=": " ");
+	   if( no < 0 )
+	      fprintf(list_fd,"%7s","");
+	   else
+	      fprintf(list_fd,"%-7d",no);
+	   fprintf(list_fd,"   %4s  %4s %-9s",
+		   IO_access[io_unit_info[i].io_access],
+		   IO_form[io_unit_info[i].io_form],
+		   keytok_name(op));
+	   old_id = id;
+	   old_no = no;
+	   old_op = op;
+	   old_acc = io_unit_info[i].io_access;
+	   old_form = io_unit_info[i].io_form;
+	   cols = 43;		/* amount of stuff printed so far */
+	}
+				/* print line number(s) after info */
+	if(cols+7 > wrap_column) {
+	   fprintf(list_fd,"\n%43s","");
+	   cols = 43;
+	}
+	fprintf(list_fd,"%6d ",
+		io_unit_info[i].line_num);
+	cols = cols+7;
+      }
+}
 
+/* Routine to sort the io_unit_usages array.  It uses bubble sort
+ * since the array is normally not very long and should be nearly
+ * sorted already in most cases.
+ */
 
+PRIVATE void
+sort_io_unit_usages(VOID)
+{
+   int i, j;
 
+   for(i = 0; i < num_io_unit_usages-1; i++) {
+      int numswaps=0;
+      for(j = num_io_unit_usages-2; j >= i; j--) {
+	 if( cmp_io_units(&io_unit_info[j],&io_unit_info[j+1]) > 0 ) {
+	    IO_Unit_Info hold = io_unit_info[j+1];
+	    io_unit_info[j+1] = io_unit_info[j];
+	    io_unit_info[j] = hold;
+            numswaps++;
+	 }
+      }
+      if(numswaps == 0)
+	 break;
+   }
+}
 
+/* Function to determine ordering of two I/O unit info entries.  The
+ * primary sort key is the unit number or unit id.  If unit number is
+ * known, it takes precedence, and sort is in numeric order.
+ * Otherwise sort is alphabetic by unit id, with the default unit=*
+ * treated as an id that comes before all alphabetic names.  If unit
+ * number or id compares equal, then sort is by operation, access and
+ * form.  (It should be rare for access and form to differ if
+ * operation is the same, but the possibility cannot be ruled out.)
+ * Line number is not used as a sort key since list arrives in
+ * line-number order and bubble sort preserves initial order.
+ */
 
+PRIVATE int
+cmp_io_units( IO_Unit_Info *u1, IO_Unit_Info *u2 )
+{
+   int no1 = u1->unit_no;
+   int no2 = u2->unit_no;
+   if( no1 == no2 ) { /* same unit no or both unknown */
+			/* if unit no unknown, compare unit id name  */
+      if( no1 == IO_UNIT_UNKNOWN ) {
+	 int id1 = u1->unit_id;
+	 int id2 = u2->unit_id;
+	 if( id1 == id2 ) {	/* same name: sort by operation, form, access */
+	    if( u1->io_operation != u2->io_operation )
+	       return u1->io_operation - u2->io_operation;
+	    else if( u1->io_access != u2->io_access )
+	       return u1->io_access - u2->io_access;
+	    else
+	       return u1->io_form - u2->io_form;
+	 }
+	 else {			/* different names: sort by name */
+	    if( id1 == IO_UNIT_DEFAULT ) {
+	       return -1;	/* '*' precedes all names */
+	    }
+	    else if( id2 == IO_UNIT_DEFAULT ) {
+	       return 1;
+	    }
+	    else if( id1 >= 0 && id2 >= 0 ) { /* they have names: compare */
+	       return strcmp(hashtab[id1].name,hashtab[id2].name);
+	    }
+	    else {
+	       return id1 - id2; /* shouldn't happen */
+	    }
+	 }
+      }
+      else {			/* if same unit no, sort by operation */
+	    if( u1->io_operation != u2->io_operation )
+	       return u1->io_operation - u2->io_operation;
+	    else if( u1->io_access != u2->io_access )
+	       return u1->io_access - u2->io_access;
+	    else
+	       return u1->io_form - u2->io_form;
+      }
+   }
+   else {			/* diff unit nos */
+      if( no1 == IO_UNIT_UNKNOWN ) {
+	 return 1;	/* unit ids follow unit nos */
+      }
+      else if( no2 == IO_UNIT_UNKNOWN ){
+	 return -1;
+      }
+      else {
+	 return no2 < no1;
+      }
+   }
+}
