@@ -1,4 +1,4 @@
-/*  $Id: project.c,v 1.11 2001/09/14 00:38:17 moniot Exp $
+/*  $Id: project.c,v 1.12 2002/08/24 15:54:17 moniot Rel $
 
     project.c:
 
@@ -487,6 +487,12 @@ proj_file_in(fd)
 
   proj_line_num = 1;
 
+  /* Allow project file to contain (manually) concatenated project files.
+     These will be processed as if the separate project files were
+     sequentially provided as arguments.
+  */
+ do {
+
  while( (retval=READ_FIRST_STR(PROJFILE_COOKIE,buf)) == 1) {
    if( strcmp(buf,PROJECT_VERSION) != 0 ) {
      (void)fprintf(stderr,
@@ -542,9 +548,11 @@ proj_file_in(fd)
 
  }/* end while(retval == 1) */
 
- if(retval != EOF) READ_ERROR;
-
  init_symtab();		/* Clear out local strspace */
+
+ /* End of a logical project file.  Continue to read any others
+    concatenated together. */
+ } while(retval != EOF);
 }
 
 static char *prev_file_name="";/* used to reduce number of callocs */
