@@ -1,4 +1,4 @@
-/* $Id: symtab.c,v 1.42 2003/03/26 01:09:22 moniot Exp $
+/* $Id: symtab.c,v 1.45 2003/08/07 19:37:48 moniot Exp $
 
   Definitions of symbol table maintenance routines and
   hash table functions
@@ -125,7 +125,52 @@ PROTO(extern void print_sizeofs,( void ));	/* in symtab.c */
 #endif
 
 
+/* Apply an attribute to a variable */
+/* N.B. Legality checking deferred to END */
+/* N. extra B.: allocatable, pointer, and target flags ignored at present */
 
+void
+apply_attr(Token *id,		/* token of variable to apply attr to */
+	   int attr)		/* token class of attr to apply */
+{
+	int h=id->value.integer;
+	Lsymtab *symt;
+
+	if( (symt=hashtab[h].loc_symtab) == NULL) {
+	   symt = install_local(h,type_UNDECL,class_VAR);
+	   symt->line_declared = id->line_num;
+	   symt->file_declared = inctable_index;
+	}
+
+	/* macro to set an attribute bit in symtab: if already set, flag
+	   the error, otherwise set the bit in all equivalenced entries.
+	 */
+#define check_and_set_attr( ATTRBIT ) if( (symt->ATTRBIT) ) { \
+	     syntax_error(id->line_num,id->col_num,"redundant"); \
+	     msg_tail(keytok_name(attr)); msg_tail("declaration"); } \
+	else {  Lsymtab *equiv=symt; \
+	  do{ \
+	    equiv->ATTRBIT = TRUE; \
+	    equiv = equiv->equiv_link; \
+	  } while(equiv != symt); }
+
+	switch( attr )
+	{
+	  case tok_ALLOCATABLE:
+	       check_and_set_attr(allocatable);
+	       break;
+	  case tok_POINTER:
+	       check_and_set_attr(pointer);
+	       break;
+	  case tok_SAVE:
+	       check_and_set_attr(saved);
+	       break;
+	  case tok_TARGET:
+	       check_and_set_attr(target);
+	       break;
+	}
+#undef check_and_set_attr
+}
 			/* This routine handles the saving of arg lists which
 			   is done by call_func and call_subr.  Also called
 			   by def_namelist to save its variable list. */
@@ -605,7 +650,6 @@ declare_type(id,datatype,size,size_text)
 	}
 }/*declare_type*/
 
-
 void
 #if HAVE_STDC
 def_arg_name(Token *id)		/* Process items in argument list */
@@ -880,6 +924,8 @@ def_com_variable(id)		/* Process items in common block list */
 
 	/* This guy sets the flag in symbol table saying the id is the
 	   current module.  It returns the hash code for later reference.
+	   Also bookmarks the source line so the declaration can be found
+	   in src buffer (currently only used by mkhtml).
 	 */
 int
 #if HAVE_STDC
@@ -892,6 +938,8 @@ def_curr_module(id)
 	int hashno = id->value.integer;
 	hashtab[hashno].loc_symtab->is_current_module = TRUE;
 
+	mark_module_srcline(id->line_num);	/* save mkhtml_bookmark */
+
 	return hashno;
 }/*def_curr_module*/
 
@@ -2169,36 +2217,6 @@ save_com_block(id)	/* Process SAVEing of a common block */
 	  symt->saved = TRUE;
 }
 
-void
-#if HAVE_STDC
-save_variable(Token *id)	/* Process SAVEing of a variable */
-	          	/* N.B. Legality checking deferred to END */
-#else /* K&R style */
-save_variable(id)	/* Process SAVEing of a variable */
-	Token *id;	/* N.B. Legality checking deferred to END */
-#endif /* HAVE_STDC */
-{
-	int h=id->value.integer;
-	Lsymtab *symt;
-
-	if( (symt=hashtab[h].loc_symtab) == NULL) {
-	   symt = install_local(h,type_UNDECL,class_VAR);
-	   symt->line_declared = id->line_num;
-	   symt->file_declared = inctable_index;
-	}
-
-	if(symt->saved) {
-	  syntax_error(id->line_num,id->col_num,
-		       "redundant SAVE declaration");
-	}
-	else {		/* set flags for all equivalenced vars */
-	  Lsymtab *equiv=symt;
-	  do{
-	    equiv->saved = TRUE;
-	    equiv = equiv->equiv_link;
-	  } while(equiv != symt);
-	}
-}
 
 	/* Following routine sets the implicit typing of characters in
 	   range c1 to c2 to the given type. */
