@@ -1,4 +1,4 @@
-/* $Id: symtab.h,v 1.36 2001/11/03 00:50:53 moniot Exp $
+/* $Id: symtab.h,v 1.43 2003/08/07 19:28:50 moniot Exp $
 
 	Shared declarations for symbol-table routines.  Note: uses
 	declarations in ftnchek.h.
@@ -610,6 +610,9 @@ typedef struct lSymtEntry{
 	     external: 1,	/* function or subr called by this routine */
 	     intrinsic: 1,	/* intrinsic function */
 	     saved: 1,		/* named in SAVE statement */
+	     allocatable: 1,	/* has ALLOCATABLE attribute */
+	     pointer: 1,	/* has POINTER attribute */
+	     target: 1,		/* has TARGET attribute */
 	     invoked_as_func: 1, /* usage as f(x) was seen */
 	     defined_in_include: 1, /* to suppress some warnings if unused */
 	     declared_external: 1, /* explicitly declared external */
@@ -632,7 +635,7 @@ typedef struct gSymtEntry{	/* Global symbol table element */
 	     used_flag: 1,
 	     set_flag: 1,
 	     assigned_flag: 1,
-	     used_before_set: 1,
+	     recursive: 1,
 	     library_module: 1,
 	     internal_entry: 1,	/* entry point other than at the top */
 	     invoked_as_func: 1,
@@ -668,6 +671,48 @@ SYM_SHARED
 #endif
 ;
 
+				/* Symbolic names for I/O access modes */
+typedef enum {
+     IO_ACCESS_DEFAULT, IO_ACCESS_DIRECT, IO_ACCESS_SEQUENTIAL
+} IO_ACCESS_TYPE;
+
+				/* Symbolic names for I/O forms */
+typedef enum {
+     IO_FORM_DEFAULT, IO_FORM_UNFORMATTED, IO_FORM_FORMATTED
+} IO_FORM_TYPE;
+
+#define IO_UNIT_UNKNOWN -1
+#define IO_UNIT_DEFAULT -2	/* For unit=* */
+				/* Struct for I/O unit usage */
+typedef struct {
+     int line_num;		/* location of I/O usage */
+     int unit_no;		/* unit number if known, else UNKNOWN */
+     int unit_id;		/* hash num of unit if variable, else UNKNOWN or DEFAULT */
+     IO_ACCESS_TYPE io_access;	/* access mode of file */
+     IO_FORM_TYPE io_form;	/* form specified for file */
+     int io_operation;		/* input, output, open, close, etc. */
+} IO_Unit_Info;
+
+SYM_SHARED
+IO_Unit_Info* io_unit_info	/* Array of I/O usage instances */
+#ifdef SYMTAB
+ = (IO_Unit_Info*)NULL
+#endif
+;
+
+SYM_SHARED
+int max_io_unit_usages		/* current size of I/O usage array */
+#ifdef SYMTAB
+ = 0
+#endif
+;
+
+SYM_SHARED
+int num_io_unit_usages		/* number of I/O usage instances in list */
+#ifdef SYMTAB
+ = 0
+#endif
+;
 				/* Struct for include-file list */
 typedef struct {
     char *fname;		/* name of include file */
@@ -876,6 +921,7 @@ StrSpace srctextspace;/* String space for token source text */
 PROTO(void assignment_stmt_type,( Token *term1, Token *equals, Token *term2 ));
 PROTO(void binexpr_type,( Token *term1, Token *op, Token *term2, Token
 		  *result ));
+PROTO(void check_initializer_type, ( Token *assignee_list, Token *equals, Token *expr_list));
 PROTO(void func_ref_expr,( Token *id, Token *args, Token *result ));
 PROTO(void primary_id_expr,( Token *id, Token *primary ));
 PROTO(void stmt_fun_arg_cmp,( const Lsymtab *symt, const Token *d_arg, const Token *a_arg ));
@@ -885,6 +931,7 @@ PROTO(int intrins_arg_cmp,( IntrinsInfo *defn, Token *t));
 
 			/* in advance.c */
 PROTO(int see_double_colon,( void ));
+PROTO(void mark_module_srcline,( LINENO_t line_num ));
 
 			/* in forlex.c */
 PROTO(void implied_id_token,( Token *t, char *s ));
@@ -899,10 +946,14 @@ PROTO(void check_seq_header,( Token *t ));
 			/* in prlocsym.c */
 PROTO(void print_loc_symbols,( void ));
 
+			/* in makehtml.c */
+PROTO(void make_html,(Lsymtab **sym_list, char *mod_name, Lsymtab *module ));
+
 			/* in makedcls.c */
 PROTO(void make_declarations,( Lsymtab *sym_list[], char *mod_name ));
 
 			/* in symtab.c */
+PROTO(void apply_attr,( Token *id, int attr ));
 PROTO(void call_func,( Token *id, Token *arg ));
 PROTO(void call_subr,( Token *id, Token *arg ));
 PROTO(char * char_expr_value,( Token *t ));
@@ -949,12 +1000,13 @@ PROTO(void print_expr_tree,( Token *t ));
 PROTO(void print_expr_list,( Token *t ));
 #endif
 PROTO(void process_lists,( int curmodhash ));
+PROTO(void record_io_unit_id, (Token *id));
+PROTO(void record_io_usage, (Token *stmt));
 PROTO(void ref_array,( Token *id, Token *subscrs ));
 PROTO(void ref_namelist,( Token *id, int stmt_class ));
 PROTO(void ref_identifier,( Token *id ));
 PROTO(void ref_variable,( Token *id ));
 PROTO(void save_com_block,( Token *id ));
-PROTO(void save_variable,( Token *id ));
 PROTO(void set_implicit_type,( int type, long size, char *len_text, int c1, int c2 ));
 PROTO(void stmt_function_stmt,( Token *id ));
 PROTO(char * token_name,( Token *t ));
