@@ -15,5 +15,12 @@ c  VAX-compatible fortran: underscores & do-enddo
       dowhile ( x .ne. (1,2))
          x = 3.0
       end do
+      do 100 while (.true.)
+         read(*,*) sum
+         if( sum .eq. 0.0 ) exit
+ 100  continue
+      do 200, while (.false.)
+         write(*,*) 'Cannot happen'
+ 200  end do
       end
       include 'average.f'
